'use strict';

var debug = require('debug')('feathers-authentication-keystone');
var merge = require('lodash.merge');
var omit = require('lodash.omit');
var pick = require('lodash.pick');
var DefaultVerifier = require('./verifier');

var _require = require('./passport'),
    Strategy = _require.Strategy;

var defaults = {
  name: 'keystone',
  usernameField: 'username',
  passwordField: 'password'
};

var INCLUDE_KEYS = ['authUrl', 'service', 'entity', 'usernameField', 'passwordField', 'passReqToCallback'];

var init = function init() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  return function keystoneAuth() {
    var app = this;
    var _super = app.setup;

    if (!app.passport) {
      throw new Error('Can not find app.passport. Did you initialize feathers-authentication before feathers-authentication-keystone?');
    }

    var name = options.name || defaults.name;
    var authOptions = app.get('authentication') || {};
    var keystoneOptions = authOptions[name] || {};

    var keystoneSettings = merge({}, defaults, pick(authOptions, INCLUDE_KEYS), keystoneOptions, omit(options, ['Verifier']));

    if (typeof keystoneSettings.authUrl === 'undefined') {
      throw new Error('You must provide an \'authUrl\' in your authentication configuration');
    }

    var Verifier = DefaultVerifier;

    if (options.Verifier) {
      Verifier = options.Verifier;
    }

    app.setup = function () {
      var result = _super.apply(this, arguments);
      var verifier = new Verifier(app, keystoneSettings);

      if (!verifier.verify) {
        throw new Error('Your verifier must implement a \'verify\' function. It should have the same signature as a keystone passport verify callback.');
      }

      debug('Registering keystone authentication strategy with options:', keystoneSettings);
      app.passport.use(keystoneSettings.name, new Strategy(keystoneSettings, verifier.verify.bind(verifier)));
      app.passport.options(keystoneSettings.name, keystoneSettings);

      return result;
    };
  };
};

init.Verifier = DefaultVerifier;

module.exports = init;